/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Aug 21, 2002 4:38:28 PM
 */
package fastforward;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Iterator;
import java.util.Vector;

public abstract class AbstractConfiguration implements IConfiguration {
    private static final String encoding = "UTF-8";

    protected AppContextImpl appContext;
    private Vector reports = new Vector();
    private ParameterSet parameters = new ParameterSet();
    private NavigationTree navigationTree;

    public void init(AppContextImpl appContext) {
        this.appContext = appContext;
    }

    public AppContextImpl getAppContext() {
        return appContext;
    }

    protected void setReports(Vector reports) {
        this.reports = reports;
    }

    protected void setParameters(ParameterSet parameters) {
        this.parameters = parameters;
    }

    protected void setNavigationTree(NavigationTree navigationTree) {
        this.navigationTree = navigationTree;
    }

    protected void readConfig(Document xmlDocument) throws XmlParseException {
        reports.clear();
        parameters.clear();
        navigationTree = new NavigationTree();

        XMLConfiguration xmlConfig = new XMLConfiguration();
        xmlConfig.init(this,appContext.getResourceBundle() );
        xmlConfig.updateConfig(xmlDocument);
    }

    public Vector getReports() {
        return reports;
    }

    public ReportImpl getReport(String reportName) {
        Iterator reportList = reports.iterator();
        while (reportList.hasNext()) {
            ReportImpl report = (ReportImpl) reportList.next();
            if (reportName.equals(report.getId()))
                return report;
        }
        return null;
    }

    public void setParameter(String parameterName, String parameterValue) {
        parameters.setParameter(parameterName, parameterValue);
    }

    public void setParameter(String parameterName, String[] parameterValues) {
        throw new IllegalArgumentException(appContext.getString("ERROR_SET_PARAM_NOT_SUPPORTED"));
    }

    public String getParameter(String parameterName) {
        return parameters.getParameter(parameterName);
    }

    public String getParameter(String parameterName, String defaultValue) {
        return parameters.getParameter(parameterName, defaultValue);
    }

    public String[] getParameterValues(String parameterName) {
        return parameters.getParameterValues(parameterName);
    }

    public String[] getParameterNames() {
        return parameters.getParameterNames();
    }

    public NavigationTree getNavigationTree() {
        return navigationTree;
    }

    public void deleteReport(String id) {
        ReportImpl report = getReport(id);
        if (report == null)
            throw new IllegalStateException(appContext.getString("ERROR_DELETE_MISSING",new Object[]{id}));

        reports.remove(report);
        NavigationTree.Report reportNode = navigationTree.findItem(id);
        reportNode.delete();
    }

    public TemplateImpl getTemplate(String templateName) throws TemplateException {
        return appContext.getTemplate(templateName);
    }

    public ParameterSet getParameterSet() {
        return parameters;
    }

    public void addReport(ReportImpl report) {
        reports.addElement(report);
    }

    protected String addDocumentDTD(Document doc, String type, String dtd) {
        OutputFormat outputFormat = new OutputFormat();

        outputFormat.setIndenting(true);
        // This is the only way we've found to disable line wrapping
        outputFormat.setLineWidth(Integer.MAX_VALUE);
        ByteArrayOutputStream stream = new ByteArrayOutputStream();

        XMLSerializer serializer = new XMLSerializer(stream, outputFormat);
        try {
            serializer.serialize(doc);
        }
        catch (IOException e) {
            throw new IllegalStateException(e.getLocalizedMessage());
        }

		String xml;

		try {
			xml = new String(stream.toByteArray(), encoding);
		}
		catch(UnsupportedEncodingException usee) {
			throw new RuntimeException(usee);
		}

        // find the DOCTYPE
        int docStart = xml.indexOf("?>");
        if (docStart == -1)
            throw new IllegalStateException(appContext.getString("ERROR_MISSING_XML_HEADER") );
        docStart += 2;
        int docEnd = xml.indexOf("<", docStart);


        StringBuffer out = new StringBuffer(xml.length());
        out.append(xml.substring(0, docStart));
        out.append("<!DOCTYPE ").append(type).append(" PUBLIC \"\" \"").append(dtd).append("\">");
        out.append(xml.substring(docEnd));
        return out.toString();
    }
}
