/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Sep 16, 2002 12:36:05 AM
 */
package fastforward;

import com.alphablox.blox.BloxContext;
import com.alphablox.blox.ContainerBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.form.FormEvent;
import com.alphablox.blox.form.FormEventListener;
import com.alphablox.blox.form.IFormBlox;
import com.alphablox.blox.form.TreeFormBlox;
import com.alphablox.blox.uimodel.BloxModel;
import com.alphablox.blox.uimodel.ModelConstants;
import com.alphablox.blox.uimodel.core.*;
import com.alphablox.blox.uimodel.core.event.ModelEvent;
import com.alphablox.server.Server;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;

public class AdminControl implements FormEventListener, TreeFormBlox.ItemDraggedEventListener {
    private AppContextImpl appContext;
    private AppConfiguration appConfig;
    private NavigationTree navigationTree;
    private TreeFormBlox treeBlox;
    private ContainerBlox adminButtons;
    private NavigationTree.Report selectedItem;
    private ReportImpl currentReport;
    private TemplateImpl currentTemplate;
    private PreviewReport previewReport;

    public AdminControl() {
    }

    public void init(AppContextImpl appContext) {
        this.appContext = appContext;
        appConfig = appContext.getAppConfiguration();
        navigationTree = appConfig.getNavigationTree();
    }

    public TreeFormBlox getTreeBlox() {
        return treeBlox;
    }

    public void setTreeBlox(TreeFormBlox treeBlox) {
        this.treeBlox = treeBlox;

        if (treeBlox != null) {
            treeBlox.addFormEventListener(this);
            treeBlox.addItemDraggedEventListener(this);
            updateAdminTree();
            try {
                treeBlox.changed();
            } catch (ModelException e) {
                appContext.log(LogFile.ERROR, appContext.getString("ERROR_INITIALIZING_TREEBLOX"), e);
                throw new IllegalStateException(appContext.getString("ERROR_UPDATING_TREE"));
            }
        }
    }

    public void valueChanged(FormEvent event) throws Exception {
        TreeFormBlox.Item treeItem = treeBlox.getSelectedItem();
        NavigationTree.Report report = null;

        if (treeItem != null)
            report = navigationTree.find(treeItem.getName());

        setSelectedItem(report);
        clearPreviewReport();

        displayEditor();
        configureButtons();
    }

    public void itemDragged(TreeFormBlox.ItemDraggedEvent event) {
        try {
            moveItem(event.getItem().getName(), event.getToParent().getName(), event.getPosition());
        } catch (Exception exc) {
            handleWriteException(exc, appContext.getString("IO_MOVING"));
        }
    }

    public void displayError(String message) {
        try {
            MessageBox errorDialog = new MessageBox(message, "Error", MessageBox.MESSAGE_OK, null);
            showDialog(errorDialog);
        } catch (Exception e) {
            appContext.log(LogFile.ERROR, appContext.getString("ERROR_DISPLAYING_DIALOG"), e);
        }
    }

    public ContainerBlox getAdminButtons() {
        return adminButtons;
    }

    public void setAdminButtons(ContainerBlox adminButtons) throws Exception {
        this.adminButtons = adminButtons;
        BloxModel model = adminButtons.getBloxModel();
        ComponentContainer buttons = (ComponentContainer) ModelResource.loadResource(appContext.getResourcePath("buttons.xml"), appContext.getLocales() );
        model.add(buttons);
        buttons.setController(new ButtonController());
        configureButtons();
        model.changed();
    }

    public void updateAppParams(String title, String logoFileName, byte[] logoContents, String theme, String footer, String emailAddress) throws Exception {
        try {
            if (logoFileName != null && logoContents != null)
                saveImageFile(logoFileName, logoContents);
            appContext.setTitle(title);
            if (logoFileName != null) // if no logo given, keep the original
                appContext.setLogo(logoFileName);
            appContext.setTheme(theme);
            appContext.setFooter(footer);
            appContext.setAdminEmail(emailAddress);

            appConfig.saveConfig();
        } catch (Exception exc) {
            reloadConfig();
            handleWriteException(exc, appContext.getString("IO_UPDATING_APP"));
        }
    }

    public void createFolder(NavigationTree.Folder parent) throws Exception {
        NavigationTree.Folder container = navigationTree.createNewFolder(parent);

        container.setLabel(appContext.getString("NEW_FOLDER_NAME"));
        container.setDescription("");

        Acl acl = new Acl(parent.getAcl());
        container.setAcl(acl);

        saveAndUpdate(container, parent);
    }

    public void deleteFolder(NavigationTree.Folder folder) throws Exception {
        Iterator itemList = folder.collectItems().iterator();

        while (itemList.hasNext()) {
            NavigationTree.Report report = (NavigationTree.Report) itemList.next();

            appConfig.deleteReport(report.getId());
        }

        folder.delete();

        saveAndUpdate(folder.getParent(), folder);
    }

    public void updateFolder(String label, String description, String[] groups) throws Exception {
        selectedItem.setLabel(label);
        selectedItem.setDescription(description);

        ((NavigationTree.Folder) selectedItem).getAcl().setGroups(groups);
        try {
            saveAndUpdate(selectedItem, selectedItem);
        } catch (Exception e) {
            handleWriteException(e, appContext.getString("IO_UPDATING_FOLDER"));
        }
    }

    /**
     * create a report given all the necessary parameters
     * @param reportData
     * @throws Exception
     */
    public String createReport(ReportData reportData) throws Exception {
        String folderID;

        if (selectedItem == null)
            folderID = navigationTree.getNavigationRoot().getId();
        else if (selectedItem.isFolder())
            folderID = selectedItem.getId();
        else
            folderID = selectedItem.getParent().getId();

        NavigationTree.Folder parent = navigationTree.findFolder(folderID);

        NavigationTree.Report reportNode = navigationTree.createNewItem(parent);

        currentReport.setId(reportNode.getId());
        currentReport.setConfig(appConfig);
        currentReport.setOwner(appContext.getUser().getName());

        appConfig.addReport(currentReport);

        try {
            updateReport(reportData);
        } catch (Exception e) {
            currentReport.setId(null); // reset the currentReport
            throw e;
        }

        return currentReport.getId();
    }

    private void saveAndUpdate(NavigationTree.Report newSelection, NavigationTree.Report oldSelection) throws Exception {
        try {
            appConfig.saveConfig();
        } catch (Exception e) {
            reloadConfig();
            // when the create report/folder fails, we would like to keep the screen unchanged
            if (navigationTree.find(oldSelection.getId()) != null)
                updateUI(oldSelection);
            throw e;
        }

        updateUI(newSelection);

    }

    private void reloadConfig() throws Exception {
        appConfig.readConfig();
        navigationTree = appConfig.getNavigationTree();
    }

    private void updateUI(NavigationTree.Report report) {
        updateAdminTree();
        appContext.getUIControl().updateTree(appConfig);

        setSelectedItem(report);
        clearPreviewReport();

        if (selectedItem == null)
            treeBlox.setSelected((String) null);
        else
            treeBlox.setSelected(selectedItem.getId());

        try {
            treeBlox.changed(); // this will cause valueChanged() event if the tree selection has really changed
        } catch (Exception e) {
            appContext.log(LogFile.ERROR, appContext.getString("ERROR_UPDATEUI"), e);
            throw new IllegalStateException(appContext.getString("ERROR_UPDATING_TREE"));
        }
    }

    private void updateAdminTree() {
        if (treeBlox == null)
            return;

        TreeFormBloxHelper helper = new TreeFormBloxHelper();

        try {
            TreeFormBlox.Folder root = helper.createAdminNavTree(appConfig,appContext.getResourceBundle());
            root.setExpanded(true);
            treeBlox.setRoot(root);
            treeBlox.setRootVisible(true);
            treeBlox.setItemPositioningEnabled(true);
        } catch (ModelException exc) {
            appContext.log(LogFile.ERROR, appContext.getString("ERROR_UPDATEADMINTREE"), exc);
            throw new IllegalStateException(appContext.getString("ERROR_UPDATING_TREE"));
        }
    }

    public void deleteReport(NavigationTree.Report reportNode) throws Exception {
        ReportImpl report = appConfig.getReport(reportNode.getId());

        // It's possible that this is a new report that was never saved
        if (report != null)
            appConfig.deleteReport(report.getId());

        reportNode.delete();

        saveAndUpdate(reportNode.getParent(), reportNode);
    }

    public boolean validateAndPreviewReport(String reportLabel, String reportDescription) throws Exception {
        ReportData testData = new ReportData(currentTemplate, reportLabel, reportDescription, getFormBlox(), appContext);

        clearPreviewReport();

        if (!validateReport(testData))
            return false;

        previewReport = new PreviewReport(currentTemplate, currentReport.getId(), reportLabel, reportDescription, getFormBlox());

        return true;
    }

    public PreviewReport getPreviewReport() {
        return previewReport;
    }

    public void clearPreviewReport() {
        previewReport = null;
    }

    public boolean validateAndSaveReport(String reportLabel, String reportDescription) throws Exception {
        ReportData reportData = new ReportData(currentTemplate, reportLabel, reportDescription, getFormBlox(), appContext);

        if (!validateReport(reportData))
            return false;

        try {
            if (currentReport.getId() == null)
                createReport(reportData);
            else
                updateReport(reportData);

        } catch (Exception e) {
            handleWriteException(e, appContext.getString("IO_SAVING"));
            return false;
        }
        return true;
    }

    private boolean validateReport(ReportData reportData) throws Exception {
        ReportValidator validator = currentTemplate.getValidator();
        String[] errors;

        if (validator == null)
            return true;

        try {
            validator.validate(reportData);
            errors = reportData.getErrors();

            if (errors == null)
                return true;

        } catch (Exception exc) {
            String text = appContext.getString("ERROR_REPORT_VALIDATION_DETAIL",new Object[]{reportData.getTemplate().getName()});
            String message = (exc.getMessage() != null) ? exc.getMessage() : exc.getClass().getName();
            appContext.log(LogFile.ERROR, text, exc);
            errors = new String[]{text, message};
        }

        MessageBox errorDialog = getErrorDialog(appContext.getString("REPORT_VALIDATION_ERROR"), errors);

        IFormBlox[] formBlox = reportData.getFormBlox();
        if (formBlox.length > 0 && formBlox[0] instanceof ContainerBlox)
            ((ContainerBlox) formBlox[0]).getBloxModel().getDispatcher().showDialog(errorDialog);
        else
            showDialog(errorDialog);

        return false;
    }

    private MessageBox getErrorDialog(String title, String[] errors) throws Exception {
        MessageBox errorDialog = new MessageBox("", title,
                MessageBox.MESSAGE_OK, null);

        Static oldStatic = errorDialog.getStatic("message");
        ComponentContainer container = oldStatic.getParent();
        container.remove(oldStatic);

        for (int x = 0; x < errors.length; x++) {
            Static message = new Static(errors[x]);
            message.setThemeClass(ModelConstants.THEME_STYLE_DIALOG_ALERT_TXT);
            container.add(message);
        }

        return errorDialog;
    }

    private IFormBlox[] getFormBlox() {
        BloxContext bloxContext = appContext.getBloxContext();
        Parameter[] params = currentTemplate.getParameters();
        Vector formBlox = new Vector();

        for (int x = 0; x < params.length; x++) {
            Parameter param = params[x];
            IFormBlox blox = (IFormBlox) bloxContext.getBlox(param.getName());
            if (blox != null)
                formBlox.addElement(blox);
        }

        return (IFormBlox[]) formBlox.toArray(new IFormBlox[formBlox.size()]);
    }

    private void updateReport(ReportData reportData) throws Exception {
        String reportID = currentReport.getId();
        NavigationTree.Report reportNode = navigationTree.findItem(reportID);

        reportNode.setLabel(reportData.getReportName());
        reportNode.setDescription(reportData.getDescription());

        ReportImpl report = appConfig.getReport(reportID);

        report.setTemplate(currentTemplate);

        // this keeps the report in memory in sync
        report.setLabel(reportData.getReportName());
        report.setDescription(reportData.getDescription());

        report.clearParameters();

        Parameter[] parameters = currentTemplate.getParameters();
        for (int x = 0; x < parameters.length; x++) {
            Parameter parameter = parameters[x];

            report.setParameter(parameter.getName(),
                    reportData.getParameterValues(parameter.getName()));
        }

        saveAndUpdate(reportNode, reportNode);
    }

    private void handleWriteException(Exception e, String context) {
        appContext.log(LogFile.ERROR, appContext.getString("ERROR_ADMIN_IO", new Object[]{context}), e);
        if (e instanceof FileNotFoundException)
            displayError(appContext.getString("ERROR_FILENOTFOUND",new Object[]{context}));
        else
            displayError(appContext.getString("ERROR_LOGTRACEBACK",new Object[]{ context,e.getMessage()}));
    }

    public void moveItem(String itemID, String folderID, int position) throws Exception {
        NavigationTree navigationTree = appConfig.getNavigationTree();
        NavigationTree.Report item = navigationTree.find(itemID);
        NavigationTree.Folder parent = navigationTree.findFolder(folderID);

        navigationTree.move(item, parent, position);

        saveAndUpdate(item, item);
    }

    public void copyReport(NavigationTree.Report report) throws Exception {
        ReportImpl reportNode = appConfig.getReport(report.getId());

        NavigationTree.Report newItem = navigationTree.createNewItem(report.getParent());
        ReportImpl newReport = new ReportImpl();
        newReport.setId(newItem.getId());
        newReport.setDescription(report.getDescription());
        newReport.setLabel(appContext.getString("COPY_OF",new Object[]{report.getLabel()}));

        reportNode.copyTo(newReport);

        appConfig.addReport(newReport);

        newItem.setLabel(appContext.getString("COPY_OF",new Object[]{report.getLabel()}));
        newItem.setDescription(report.getDescription());

        saveAndUpdate(newItem, report);
    }

    private void saveImageFile(String fileName, byte[] contents) throws Exception {
        String path = appContext.getRealPath("images");
        File f = new File(path, fileName);
        if (f.exists() && !f.canWrite())
            throw new IOException(appContext.getString("ERROR_LOCKED_FILE"));

        f.getParentFile().mkdirs();

        FileOutputStream fos = new FileOutputStream(f);
        fos.write(contents);
        fos.close();
    }

    public void setSelectedItem(NavigationTree.Report selectedItem) {
        // since we reload the tree on failure, don't use the selectedItem reference as it is. find it from the tree and use it.
        this.selectedItem = (selectedItem == null) ? null : navigationTree.find(selectedItem.getId());
    }

    private void displayEditor() {
        displayItem(selectedItem);
    }

    public void clearUIReport() {
        if (appContext != null && appContext.getUIControl() != null)
            appContext.getUIControl().clearReport();
    }

    public void clearAdminReport() {
        this.selectedItem = null;
        setCurrentReport(null);
        //updateUI(this.selectedItem);
        treeBlox.setSelected((String) null);
        try {
            treeBlox.changed();
        } catch (ModelException e) {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }
    }

    private void displayNewReport() {
        clearPreviewReport();

        IModelDispatcher dispatcher;
        try {
            dispatcher = treeBlox.getBloxModel().getDispatcher();
        } catch (Exception e) {
            appContext.log(LogFile.ERROR, "Exception thrown in displayNewReport()", e);
            return;
        }

        String url = "editreport.jsp?render=" + appContext.getRenderMode();

        dispatcher.showBrowserWindow(new ClientLink(url, "ffwd_admin_main"));
    }

    private void displayItem(NavigationTree.Report item) {
        IModelDispatcher dispatcher;
        try {
            dispatcher = treeBlox.getBloxModel().getDispatcher();
        } catch (Exception e) {
            appContext.log(LogFile.ERROR, appContext.getString("ERROR_EXPECTED_EXCEPTION"), e);
            return;
        }
        String url = null;
        String params = "render=" + appContext.getRenderMode();

        if (item == null)
            url = "blank.jsp?" + params;
        else if (selectedItem.isRoot())
            url = "editroot.jsp?" + params;
        else if (selectedItem.isFolder())
            url = "editfolder.jsp?folder=" + item.getId() + "&" + params;
        else
            url = "editreport.jsp?reportId=" + item.getId() + "&" + params;

        dispatcher.showBrowserWindow(new ClientLink(url, "ffwd_admin_main"));
    }

    private void configureButtons() {
        ComponentContainer buttons;

        try {
            buttons = (ComponentContainer) adminButtons.getBloxModel().findInContainer("adminButtons");

        } catch (Exception e) {
            appContext.log(LogFile.ERROR, "Unexpected model exception", e);
            return;
        }

        ToolbarButton deleteItem = (ToolbarButton) buttons.searchForComponent("deleteItem");
        ToolbarButton copyReport = (ToolbarButton) buttons.searchForComponent("copyReport");
        ToolbarButton createReport = (ToolbarButton) buttons.searchForComponent("createReport");
        ToolbarButton createFolder = (ToolbarButton) buttons.searchForComponent("createFolder");

        if (selectedItem == null) {
            deleteItem.setDisabled(true);
            copyReport.setDisabled(true);
            createReport.setDisabled(false);
            createFolder.setDisabled(false);
        } else if (selectedItem.isRoot()) {
            deleteItem.setDisabled(true);
            copyReport.setDisabled(true);
            createReport.setDisabled(false);
            createFolder.setDisabled(false);
        } else if (selectedItem.isFolder()) {
            deleteItem.setDisabled(false);
            copyReport.setDisabled(true);
            createReport.setDisabled(false);
            createFolder.setDisabled(false);
        } else {
            deleteItem.setDisabled(false);
            copyReport.setDisabled(false);
            createReport.setDisabled(false);
            createFolder.setDisabled(false);
        }
        try {
            buttons.changed();
        } catch (ModelException e) {
            appContext.log(LogFile.ERROR, appContext.getString("ERROR_UNEXPECTED_EXCEPTION"), e);
        }
    }

    public void showDialog(Dialog dialog) throws ServerBloxException, ModelException {
        IModelDispatcher dispatcher = treeBlox.getBloxModel().getDispatcher();
        dispatcher.showDialog(dialog);
    }

    public void setCurrentReport(ReportImpl report) {
        this.currentReport = report;
    }

    public void setCurrentTemplate(TemplateImpl currentTemplate) {
        this.currentTemplate = currentTemplate;
    }

    public boolean okToDisplayAdminPage() throws Exception {
        if (appContext.isTemplateApplication()) {
            Dialog dialog = new CreateApplicationDialog();
            appContext.getUIControl().showDialog(dialog);
            return false;
        }

        return true;
    }

    public class ButtonController extends Controller {
        public void actionCreateFolder(ModelEvent event) throws Exception {
            try {
                if (selectedItem == null || (!selectedItem.isFolder()))

                    createFolder(navigationTree.getNavigationRoot());
                else
                    createFolder((NavigationTree.Folder) selectedItem);

            } catch (Exception e) {
                appContext.log(LogFile.ERROR, appContext.getString("ERROR_CREATING_FOLDER"), e);
                displayError(appContext.getString("ERROR_CREATING_FOLDER_DETAIL"));
            }
        }

        public void actionDeleteItem(ModelEvent event) throws Exception {
            if (selectedItem.isFolder()) {
                ConfirmDeleteDialog dialog = new ConfirmDeleteDialog(selectedItem,
                        appContext.getString("DELETE_FOLDER_CONFIRM"),
                        appContext.getString("DELETE_FOLDER_YN"));
                event.getComponent().getDispatcher().showDialog(dialog);
            } else {
                ConfirmDeleteDialog dialog = new ConfirmDeleteDialog(selectedItem,
                        appContext.getString("DELETE_REPORT_CONFIRM"),
                        appContext.getString("DELETE_REPORT_YN"));
                event.getComponent().getDispatcher().showDialog(dialog);
            }

        }

        public void actionCreateReport(ModelEvent event) throws Exception {
            displayNewReport();
        }

        public void actionCopyReport(ModelEvent event) throws Exception {
            try {
                copyReport(selectedItem);
            } catch (Exception e) {
                handleWriteException(e, appContext.getString("IO_COPYING"));
            }
        }

        public void actionDeleteReport(ModelEvent event) throws Exception {
            ConfirmDeleteDialog dialog = new ConfirmDeleteDialog(selectedItem,
                    appContext.getString("DELETE_REPORT_CONFIRM"),
                    appContext.getString("DELETE_REPORT_YN"));
            event.getComponent().getDispatcher().showDialog(dialog);
        }
    }

    class ConfirmDeleteDialog extends Dialog {
        private NavigationTree.Report report;

        public ConfirmDeleteDialog(NavigationTree.Report report, String title, String prompt) {
            super();
            this.report = report;
            try {
                loadFromResource(appContext.getResourcePath("confirm.xml"), appContext.getLocales() );
                setTitle(title);
                Static text = (Static) searchForComponent("prompt");
                text.setTitle(prompt);
                setController(new DialogController(this) {
                    public void actionOk() {
                        deleteItem();
                        super.actionOk();
                    }
                });
            } catch (Exception e) {
                appContext.log(LogFile.ERROR, appContext.getString("ERROR_UNEXPECTED_EXCEPTION"), e);
            }
        }

        void deleteItem() {
            try {
                if (report.isFolder())
                    deleteFolder((NavigationTree.Folder) report);
                else
                    deleteReport(report);
            } catch (Exception e) {
                handleWriteException(e, appContext.getString("IO_DELETING"));
            }
        }
    }

    class CreateApplicationDialog extends Dialog {
        public CreateApplicationDialog() {
            super();
            try {
                loadFromResource(appContext.getResourcePath("createapp.xml"), appContext.getLocales() );
                setController(new DialogController(this) {
                    public void actionCreate() {
                        createApplication();
                        super.actionOk();
                    }

                    public void actionContinue() {
                        displayAdminPage();
                        super.actionCancel();
                    }
                });
            } catch (Exception e) {
                appContext.log(LogFile.ERROR, appContext.getString("ERROR_UNEXPECTED_EXCEPTION"), e);
            }
        }

        void createApplication() {
            try {
                dispatcher.showDialog(new NewApplicationDialog());
            } catch (ModelException exc) {
                appContext.log(LogFile.ERROR, appContext.getString("ERROR_DISPLAYING_NEW_APP_DLG"), exc);
            }
        }

        void displayAdminPage() {
            String adminURL = appContext.getAppURL() + "admin/admin.jsp";
            String command = "displayAdminPage('" + adminURL + "');";
            dispatcher.sendClientCommand(appContext.getCallbackFunction(command));
        }
    }

    class NewApplicationDialog extends Dialog {
        public NewApplicationDialog() {
            super();
            try {
                loadFromResource(appContext.getResourcePath("newapp.xml"), appContext.getLocales() );

                Edit adminRole = (Edit) searchForComponent("adminRole");
                adminRole.setValue(appContext.getAdminRole());

                setController(new DialogController(this) {
                    public void actionOk() {
                        if (createApplication())
                            super.actionOk();
                    }
                });
            } catch (Exception e) {
                appContext.log(LogFile.ERROR, appContext.getString("ERROR_UNEXPECTED_EXCEPTION"), e);
            }
        }

        boolean createApplication() {
            String contextName = getValue("contextName");
            String displayName = getValue("displayName");
            String description = getValue("description");
            String adminRole = getValue("adminRole");

            Vector errors = new Vector();
            if (contextName == null)
                errors.addElement("Context Name must be provided.");

            if (displayName == null)
                errors.addElement("Display Name must be provided.");

            if (adminRole == null)
                errors.addElement("AdminRole must be provided.");

            if (errors.size() > 0) {
                try {
                    Dialog errorDialog = getErrorDialog("", (String[]) errors.toArray(new String[errors.size()]));
                    dispatcher.showDialog(errorDialog);
                } catch (Exception exc) {
                    appContext.log(LogFile.ERROR, "Error trying to display error dialog", exc);
                }
                return false;
            }

            try {
                appContext.createApplication(contextName, displayName, description, adminRole);
            } catch (Exception exc) {
                appContext.log(LogFile.ERROR, "Error trying to create a new Application", exc);
                MessageBox.message(dispatcher, "Could not create Application", exc.getMessage());
                return false;
            }

            String uri = "/" + contextName;

            ClientLink newAdminPage = new ClientLink(uri + "/admin/admin.jsp");

            MessageBox.message(dispatcher, appContext.getString("APPLICATION_CREATED"),
                    Server.getServer().getWASAdapter().getAfterActionMessage("create", contextName, contextName),
                    MessageBox.MESSAGE_OK, new ShowNewAdminPage(newAdminPage));

            return true;
        }

        private class ShowNewAdminPage implements IMessageCallback {
            private ClientLink lnk;

            public ShowNewAdminPage(ClientLink link) {
                lnk = link;
            }

            public boolean action(MessageBox messageBox, String action) {
                messageBox.close();

                if (!appContext.isIBMWebSphere())
                    dispatcher.showBrowserWindow(lnk);
                return true;
            }
        }

        private String getValue(String componentName) {
            Edit edit = (Edit) searchForComponent(componentName);

            if (edit == null)
                return null;

            String value = edit.getValue().trim();

            return (value.length() != 0) ? value : null;
        }
    }
}
