/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: rraghavan
 * Date: Sep 6, 2002
 * Time: 11:34:08 AM
 */
package fastforward;


import java.io.File;
import java.io.IOException;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.jsp.PageContext;

import com.alphablox.blox.*;
import com.alphablox.blox.form.TreeFormBlox;
import com.alphablox.blox.repository.Application;
import com.alphablox.resource.Messages;
import com.alphablox.server.Server;
import com.alphablox.server.adapter.IWASAdapter;
import com.alphablox.server.adapter.WebDescriptor;
import com.alphablox.server.application.ApplicationManager;
import com.alphablox.server.cluster.ClusterManager;
import com.alphablox.server.service.ServiceManager;
import com.alphablox.util.FileUtils;
import com.alphablox.util.Message;
import com.alphablox.util.NotFoundException;

public class AppContextImpl implements HttpSessionBindingListener, AppContext {
    public final static int MAJOR_VERSION = 1;
    public final static int MINOR_VERSION = 0;
    public final static String APP_STUDIO_PATH = "/system/ApplicationStudio/FastForward";
    public final static String APP_STUDIO_EAR = "/ApplicationStudio.ear/FastForward.war";
    public final static String ESSBASE_DATASOURCE_NAME = "QCC-Essbase";
    public final static String MSAS_DATASOURCE_NAME = "QCC-MSAS";
    public static final String TEMPLATE_DIRECTORY = "templates";
    public static final String CONFIG_FILE = "WEB-INF/config.xml";
    public static final String CONFIG_DTD_FILE = "WEB-INF/config.dtd";
    public static final String LOGFILE = "WEB-INF/fastforward.log";

    private final static String CALLBACK_HOME = "";
    private final static String CALLBACK_OBJ = CALLBACK_HOME + (CALLBACK_HOME.equals("") ? "" : ".") + "callback";
    private static final String RESOURCE_DIR = "WEB-INF/ui/";

    private AppConfiguration appConfig;
    private UserConfiguration userConfig;
    private Hashtable groupConfigMap = new Hashtable();
    private UserImpl user;
    private BloxContext bloxContext;
    private RepositoryBlox repositoryBlox;
    private BookmarksBlox bookmarksBlox;
    private ReportImpl currentReport;
    private File appDirectory;
    private LogFile logFile;
    private AdminControl adminControl;
    private UIControl uiControl;
    private TreeFormBlox reportNavigationTree;
    private FastForwardTabs reportViewSelector;
    private ContainerBlox toolbar;
    private Map templateCache = new HashMap();
    private String contextPath;
    private SavedStateImpl savedState;
    private String appURL;
    private ContainerBlox reportTitle;
    private ContainerBlox reportDescription;
    private ContainerBlox reportHelpLink;
    private ClusterManager clusterManager;

    // Application Params
    private final String RENDER_MODE = "RenderMode";
    private final String THEME = "Theme";
    private final String TITLE = "Title";
    private final String LOGO = "Logo";
    private final String FOOTER = "Footer";
    private final String ADMIN_EMAIL = "AdminEmail";
    private final String ADMIN_ROLE = "AdministratorRole";
    private ContainerBlox helpToolbar;
    private FastForwardResourceBundle resource;
    private String[] localeTargets;

    public AppContextImpl() {
    }

    /**
     * Initializes the application context -- stuff that may be referenced from several places.
     *
     * @param pageContext enables access to the servletrequest and servletresponse.
     * @throws fastforward.XmlParseException
     * @throws com.alphablox.blox.ServerBloxException
     *
     */
    public void init(PageContext pageContext) throws Exception {
        HttpServletRequest httpRequest = (HttpServletRequest) pageContext.getRequest();
        HttpServletResponse httpResponse = (HttpServletResponse) pageContext.getResponse();

        bloxContext = BloxContextFactory.getBloxContext(httpRequest, httpResponse);
        BloxRequest bloxRequest = bloxContext.registerRequest(httpRequest, httpResponse);
        BloxSession session = bloxRequest.getBloxSession();

        appDirectory = new File(pageContext.getServletContext().getRealPath(""));
        contextPath = httpRequest.getContextPath();
        Locale locale = bloxContext.getLocale();
        localeTargets = getLocaleTargets(locale);

        // create logfile
        logFile = new LogFile(session.getId(), Locale.getDefault());
        logFile.setFile(new File(getRealPath(LOGFILE)));

        // get resource bundle
        try {
            resource = new FastForwardResourceBundle(locale, pageContext.getPage().getClass().getClassLoader());
        } catch (MissingResourceException mre) {
            log(LogFile.ERROR, "FastForwardBundle.properties is missing");
            throw new IllegalStateException("FastForwardBundle.properties is missing");
        }

        String entireURL = HttpUtils.getRequestURL(httpRequest).toString();
        appURL = entireURL.substring(0, entireURL.indexOf(contextPath) + contextPath.length() + 1);

        // read configuration
        appConfig = new AppConfiguration();
        appConfig.init(this);
        appConfig.readConfig();
        String debugLevel = appConfig.getParameter("debugLevel");
        if (debugLevel != null && !debugLevel.equals("")) {
            int level;
            try {
                level = Integer.parseInt(debugLevel);
                logFile.setLogLevel(level);
            } catch (NumberFormatException e) {
                new Message(Message.WARNING, contextPath, getString("WARNING_DEFAULT_DEBUG")).send();
            }
        }
        // doing it after the LogFile is set completely

        createFFBlox();

        // create user
        user = new UserImpl();
        user.init(repositoryBlox);
        userConfig = new UserConfiguration();
        userConfig.init(this);
        userConfig.readConfig();

        // create the group config for each of the group that this user belongs to
        // and keep them in a hashtable
        String[] groups = user.getGroups();
        for (int i = 0; i < groups.length; i++) {
            String group = groups[i];
            GroupConfiguration groupConfig = new GroupConfiguration();
            groupConfig.init(this, group);
            groupConfig.readConfig();
            groupConfigMap.put(group, groupConfig);
        }

        String adminRole = getAdminRole();
        user.setIsAdministrator(bloxRequest.isUserInRole(adminRole));

        // create RMI API
        adminControl = new AdminControl();
        adminControl.init(this);
        session.setAttribute("adminControl", adminControl);

        uiControl = new UIControl();
        uiControl.init(this);
        session.setAttribute("uiControl", uiControl);

        try {
            uiControl.setTreeBlox(reportNavigationTree);
            uiControl.setNavigationSwitcher(reportViewSelector);
            uiControl.setToolbarBlox(toolbar);
            uiControl.setToolbarHelpBlox(helpToolbar);
            uiControl.setReportTitleBlox(reportTitle);
            uiControl.setReportDescriptionBlox(reportDescription);
            uiControl.setHelpLinkBlox(reportHelpLink);
        } catch (Exception e) {
            log(LogFile.ERROR, e.getMessage(), e);
        }

        savedState = new SavedStateImpl(this);

        log(LogFile.INFO, getString("INFO_SESSION_STARTED", new Object[]{user.getName()}));
    }

    public boolean isClustered() {
        return getClusterManager().getIsClustered();
    }

    protected ClusterManager getClusterManager() {
        if (clusterManager != null)
            return clusterManager;

        try {
            clusterManager = (ClusterManager) ServiceManager.getService(ServiceManager.SVC_CLUSTERMGR, null);
        } catch (NotFoundException e) {
            clusterManager = null;
        }

        return clusterManager;
    }

    public String getInstanceName() {
        return Server.getServer().getInstanceName();
    }


    private void createFFBlox() throws ServerBloxException, InvalidParameterException {
        reportNavigationTree = new TreeFormBlox();
        reportNavigationTree.init(bloxContext, "ffwdNavigationBlox");
        reportViewSelector = new FastForwardTabs(resource) {
            public boolean handleTab(String tab) {
                if (uiControl != null)
                    return uiControl.handleNewTab(tab);
                return true;
            }
        };


        reportViewSelector.init(bloxContext, "ffwdSwitchView");

        reportViewSelector.addTab(getString("PUBLIC_ACCESS"), Report.PUBLIC);
        reportViewSelector.addTab(getString("PRIVATE_ACCESS"), Report.PRIVATE);
        reportViewSelector.addTab(getString("GROUP_ACCESS"), Report.GROUP);
        reportViewSelector.setSelected(0);
        reportViewSelector.setPrefix("images/tab");
        //    reportViewSelector.setWidth("250");


        this.repositoryBlox = (RepositoryBlox) bloxContext.getBlox("ffwdRepoBlox");
        if (this.repositoryBlox == null) {
            this.repositoryBlox = new RepositoryBlox();
            this.repositoryBlox.init(bloxContext, "ffwdRepoBlox");
        }

        this.bookmarksBlox = (BookmarksBlox) bloxContext.getBlox("ffwdBookmarksBlox");
        if (this.bookmarksBlox == null) {
            this.bookmarksBlox = new BookmarksBlox();
            this.bookmarksBlox.init(bloxContext, "ffwdBookmarksBlox");
        }

        toolbar = new ContainerBlox();
        toolbar.init(bloxContext, "ffwdToolbarBlox");

        helpToolbar = new ContainerBlox();
        helpToolbar.init(bloxContext, "ffwdToolbarHelpBlox");

        reportTitle = new ContainerBlox();
        reportTitle.init(bloxContext, "ffwdReportTitleBlox");

        reportDescription = new ContainerBlox();
        reportDescription.init(bloxContext, "ffwdReportDescriptionBlox");

        reportHelpLink = new ContainerBlox();
        reportHelpLink.init(bloxContext, "ffwdReportHelpLinkBlox");
    }

    public SavedState getSavedState() {
        return savedState;
    }

    public Locale getLocale() {
        return bloxContext.getLocale();
    }

    public Locale[] getLocales() {
        return bloxContext.getLocales();
    }

    public String getAppURL() {
        return appURL;
    }

    public void setAppURL(String appURL) {
        this.appURL = appURL;
    }

    public File getConfigFile() {
        return new File(appDirectory, CONFIG_FILE);
    }

    public String getContextPath() {
        return contextPath;
    }

    public AdminControl getAdminControl() {
        return adminControl;
    }

    public TreeFormBlox getReportNavigationTree() {
        return reportNavigationTree;
    }

    public FastForwardTabs getReportViewSelector() {
        return reportViewSelector;
    }

    public ContainerBlox getToolbar() {
        return toolbar;
    }

    public ContainerBlox getHelpToolbar() {
        return helpToolbar;
    }

    public ContainerBlox getReportTitleBlox() {
        return reportTitle;
    }

    public ContainerBlox getReportDescriptionBlox() {
        return reportDescription;
    }

    /**
     * Get the repository blox associated with the application.
     *
     * @return repositoryBlox
     */
    public RepositoryBlox getRepositoryBlox() {
        return repositoryBlox;
    }

    public BookmarksBlox getBookmarksBlox() {
        return bookmarksBlox;
    }

    public AppConfiguration getAppConfiguration() {
        return appConfig;
    }

    public User getUser() {
        return user;
    }

    public Report getCurrentReport() {
        return currentReport;
    }

    public void setCurrentReport(ReportImpl currentReport) throws ServerBloxException {
        synchronized (this) {
            // Remove any blox from previously viewed report
            Blox[] oldBlox = bloxContext.getBlox();
            for (int x = 0; x < oldBlox.length; x++) {
                String bloxName = oldBlox[x].getBloxName();
                if (!bloxName.startsWith("ffwd")) {
                    oldBlox[x].deleteBlox();
                }
            }
        }

        this.currentReport = currentReport;

        uiControl.updateToolbar();
        uiControl.updateToolbarHelp();
        uiControl.updateTitleLoading();
        uiControl.clearDescription();
        uiControl.clearHelpLink();
        if (currentReport == null) {
            // Clear previous report data
            savedState.init(null, null);
            return;
        }

        savedState.init(currentReport, uiControl.getVisibility());
    }

    /**
     * Add a message to the application log file.
     *
     * @param type    message type. See LogFile class for the defined constants (DEBUG, INFO...)
     * @param message the text to append to the application's log file.
     */
    public void log(int type, String message) {
        try {
            logFile.writeln(type, message);
        } catch (IOException exc) {
            new Message(Message.ERROR, contextPath, getString("WARNING_LOG_FAILED", new Object[]{message})).send();
        }
    }

    /**
     * Add a message to the application log file.  Include information from an exception.
     *
     * @param type      message type. See LogFile class for the defined constants (DEBUG, INFO...)
     * @param message   the text to append to the application's log file.
     * @param throwable the exception
     */
    public void log(int type, String message, Throwable throwable) {
        if (message == null)
            message = throwable.getClass().getName();

        try {
            logFile.writeln(type, message, throwable);
        } catch (IOException e) {
            throwable.printStackTrace(System.err);
        }
        new Message(Message.ERROR, getString("ERROR_EXCEPTION_LOGGED", new Object[]{contextPath}), throwable).send();
    }

    /**
     * Find a localized string in the FastForwardBundle.properties file.
     *
     * @param key
     * @return null if the String is not found
     */
    public String getString(String key) {
        return resource.getString(key);
    }

    /**
     * Find a localized string in the FastForwardBundle.properties file.
     *
     * @param key
     * @param args object array for replacement
     * @return null if the String is not found
     */
    public String getString(String key, Object args[]) {
        return resource.getString(key, args);
    }

    /**
     * Get a report.
     *
     * @param reportName
     * @return ReportImpl
     * @throws fastforward.NoAccessException
     */
    public ReportImpl getReport(String reportName) throws NoAccessException {
        return getReport(reportName, Report.PUBLIC, null);
    }

    /**
     * Get a report.
     *
     * @param reportName
     * @param access
     * @param groupName
     * @return ReportImpl
     * @throws fastforward.NoAccessException
     */
    public ReportImpl getReport(String reportName, String access, String groupName) throws NoAccessException {
        ReportImpl report = null;

        if (access.equals(Report.PUBLIC))
            report = appConfig.getReport(reportName);
        else if (access.equals(Report.PRIVATE))
            report = userConfig.getReport(reportName);
        else if (access.equals(Report.GROUP)) {
            if (!user.isInGroup(groupName))
                throw new NoAccessException(Messages.NO_REPORT_ACCESS, new String[]{reportName});
            GroupConfiguration groupConfig = (GroupConfiguration) groupConfigMap.get(groupName);
            report = groupConfig.getReport(reportName);
        }
        if (report == null)
            return null;

        if (!report.canViewReport(user))
            throw new NoAccessException(Messages.NO_REPORT_ACCESS, new String[]{report.getId()});

        return report;
    }

    public TemplateImpl getTemplate(String templateName) throws TemplateException {
        if (templateCache.containsKey(templateName))
            return (TemplateImpl) templateCache.get(templateName);

        // not found in cache... reload templates from disk to see
        // if it's been recently added
        getTemplates();
        if (templateCache.containsKey(templateName))
            return (TemplateImpl) templateCache.get(templateName);

        log(LogFile.ERROR, getString("ERROR_LOADING_TEMPLATE", new Object[]{templateName}),
                new TemplateException(getString("ERROR_LOADING_TEMPLATE", new Object[]{templateName})));

        return new MissingTemplate(templateName);
    }

    public String getRealPath(String path) {
        return new File(appDirectory, path).getPath();
    }

    // We're careful here to keep any existing template objects up to date because we may
    // have reports loaded with pointers to them.
    public Template[] getTemplates() throws TemplateException {
        Template[] templates = TemplateImpl.getTemplates(new File(getRealPath(TEMPLATE_DIRECTORY)), getLocale());

        // remove old templates (in two passes)
        Iterator iterator = templateCache.keySet().iterator();
        Vector removedTemplates = new Vector();
        while (iterator.hasNext()) {
            String templateName = (String) iterator.next();
            if (!contains(templateName, templates))
                removedTemplates.addElement(templateName);
        }
        for (int x = 0; x < removedTemplates.size(); x++) {
            templateCache.remove(removedTemplates.elementAt(x));
        }

        // Add or update templates
        for (int x = 0; x < templates.length; x++) {
            Template template = templates[x];

            if (!templateCache.containsKey(template.getName()))
                templateCache.put(template.getName(), template);
            else {
                TemplateImpl existingTemplate = (TemplateImpl) templateCache.get(template.getName());
                ((TemplateImpl) template).copyTo(existingTemplate);
            }

        }

        return (Template[]) templateCache.values().toArray(new Template[templateCache.values().size()]);
    }

    private boolean contains(String templateName, Template[] templates) {
        for (int x = 0; x < templates.length; x++) {
            if (templates[x].getName().equals(templateName))
                return true;
        }
        return false;
    }

    public String getResourcePath() {
        return getResourcePath("");
    }

    public String getResourcePath(String path) {
        return getRealPath(RESOURCE_DIR + path);
    }

    public String getParameter(String parameterName) {
        return appConfig.getParameter(parameterName);
    }

    public String getParameter(String parameterName, String defaultValue) {
        return appConfig.getParameter(parameterName, defaultValue);
    }

    public String[] getParameterValues(String parameterName) {
        return appConfig.getParameterValues(parameterName);
    }

    public String[] getParameterNames() {
        return appConfig.getParameterNames();
    }

    public String getRenderMode() {
        return appConfig.getParameter(RENDER_MODE, "dhtml");
    }

    public void setRenderMode(String renderMode) {
        appConfig.setParameter(RENDER_MODE, renderMode);
    }

    public String getTheme() {
        return appConfig.getParameter(THEME, "coleman");
    }

    public void setTheme(String theme) {
        appConfig.setParameter(THEME, theme);
    }

    public String getTitle() {
        return appConfig.getParameter(TITLE, "FastForward Application");
    }

    public void setTitle(String title) {
        appConfig.setParameter(TITLE, title);
    }

    public String getLogo() {
        return appConfig.getParameter(LOGO, "alphablox.gif");
    }

    public void setLogo(String imageLocation) {
        appConfig.setParameter(LOGO, imageLocation);
    }

    public String getFooter() {
        return appConfig.getParameter(FOOTER, "Copyright 2004 IBM Corporation. All rights reserved.");
    }

    public void setFooter(String footer) {
        appConfig.setParameter(FOOTER, footer);
    }

    public String getAdminRole() {
        return appConfig.getParameter(ADMIN_ROLE, "AlphabloxAdministrator");
    }

    public String getAdminEmail() {
        return appConfig.getParameter(ADMIN_EMAIL, "");
    }

    public String getVersionString() {
        return AppContextImpl.MAJOR_VERSION + "." + AppContextImpl.MINOR_VERSION;
    }

    public void setAdminEmail(String emailAddress) {
        appConfig.setParameter(ADMIN_EMAIL, emailAddress);
    }

    public UserConfiguration getUserConfiguration() {
        return userConfig;
    }

    public GroupConfiguration getGroupConfiguration(String groupName) {
        return (GroupConfiguration) groupConfigMap.get(groupName);
    }

    public GroupConfiguration[] getGroupConfigurations() {
        Vector v = new Vector();
        String[] groups = user.getGroups();
        for (int i = 0; i < groups.length; i++) {
            GroupConfiguration groupConfig = (GroupConfiguration) groupConfigMap.get(groups[i]);
            if (groupConfig != null) {
                v.add(groupConfig);
            }
        }
        return (GroupConfiguration[]) v.toArray(new GroupConfiguration[v.size()]);
    }

    public BloxContext getBloxContext() {
        return bloxContext;
    }

    public UIControl getUIControl() {
        return uiControl;
    }

    public String getReportURL(Report report) {
        return getReportURL("report.jsp", (ReportImpl) report) + "&render=" + getRenderMode();
    }

    public String getPrintURL(Report report) {
        return getReportURL("print.jsp", (ReportImpl) report) + "&render=printer";
    }

    public String getExportToExcelURL(Report report) {
        return getReportURL("excel.jsp", (ReportImpl) report) + "&render=xls";
    }

    public String getMailURL(Report report) {
        return getReportURL("mailreport.jsp", (ReportImpl) report) + "&render=" + getRenderMode() + "&mail=true";
    }

    /**
     * Set the logging level. See LogFile.java for the available log levels.
     *
     * @param level sets the new log level
     */
    public void setLogLevel(int level) {
        logFile.setLogLevel(level);
    }

    /**
     * Gets the current log level
     *
     * @return the log level
     */
    public int getLogLevel() {
        return logFile.getLogLevel();
    }

    private String getReportURL(String page, ReportImpl report) {
        StringBuffer url = new StringBuffer(getAppURL());
        url.append(page);
        url.append("?report=").append(report.getId());
        url.append("&type=").append(report.getVisibility());
        url.append("&theme=").append(getTheme());
        if (report.getVisibility().equals(Report.GROUP))
            url.append("&group=").append(((GroupConfiguration) report.getConfig()).getGroup());

        return url.toString();
    }

    public String getLocalizedURL(String relativePath) {
        File baseFile = new File(relativePath);
        String fileName = baseFile.getName();
        String baseFileName = fileName;
        String extension = "";
        String params = "";
        int dotIndex = fileName.lastIndexOf('.');
        if (dotIndex >= 0) {
            baseFileName = fileName.substring(0, dotIndex);
            extension = fileName.substring(dotIndex);
        }

        int paramIndex = extension.indexOf('?');
        if (paramIndex >= 0) {
            params = extension.substring(paramIndex);
            extension = extension.substring(0, paramIndex);
        }

        String[] localeTargets = getLocaleTargets();

        for (int x = 0; x < localeTargets.length; x++) {
            String filePath = baseFile.getParent() + '/' + baseFileName + '_' + localeTargets[x] + extension;

            if ((new File(getRealPath(filePath))).exists())
                return filePath.replace('\\', '/') + params;
        }

        return relativePath;
    }

    public String getLocalizedURL(String subReference, String relativePath) {
        String url = getLocalizedURL(subReference + relativePath);
        return url.substring(subReference.length());
    }

    public String[] getLocaleTargets() {
        return localeTargets;
    }

    public static String[] getLocaleTargets(Locale locale) {
        StringBuffer buffer = new StringBuffer();
        String language = locale.getLanguage();
        int languageLength = language.length();
        String country = locale.getCountry();
        int countryLength = country.length();

        if (languageLength + countryLength == 0)
            return new String[0];

        ArrayList targets = new ArrayList();

        buffer.append(language);

        if (languageLength > 0)
            targets.add(buffer.toString());

        if (countryLength > 0) {
            buffer.append('_');
            buffer.append(country);
            targets.add(buffer.toString());
        }

        String[] localeTargets = new String[targets.size()];

        for (int x = 0; x < localeTargets.length; x++)
            localeTargets[x] = (String) targets.get(localeTargets.length - 1 - x);

        return localeTargets;
    }

    /**
     * Build a safe function call that the browser can execute in javascript.
     *
     * @param method is function that must exist in the callback.js file
     * @return a safe javascript string
     */
    public String getCallbackFunction(String method) {
        String ret;
        if (CALLBACK_HOME.equals("")) {
            ret = "if (typeof(" + CALLBACK_OBJ + ")!='undefined'){" + CALLBACK_OBJ + "." + method + ";}";
        } else {
            ret = "if(typeof(" + CALLBACK_HOME + ")!= 'undefined' && typeof(" + CALLBACK_OBJ + ") != 'undefined'){" + CALLBACK_OBJ + "." + method + ";}";
        }

        return ret;

    }

    public void valueBound(HttpSessionBindingEvent event) {
    }

    public void valueUnbound(HttpSessionBindingEvent event) {
        if (user != null)
            log(LogFile.INFO, getString("INFO_SESSION_ENDED", new Object[]{user.getName()}));
    }

    public boolean isTemplateApplication() {
        String appPath = appDirectory.getPath().replace('\\', '/');

        if (isIBMWebSphere())
            return appPath.endsWith(APP_STUDIO_EAR);
        else
            return appPath.endsWith(APP_STUDIO_PATH);
    }

    public int getMajorVersion() {
        return AppContextImpl.MAJOR_VERSION;
    }

    public int getMinorVersion() {
        return AppContextImpl.MINOR_VERSION;
    }

    public boolean isSampleDataSourceAvaliable() {
        String[] dataSources;
        try {
            dataSources = getRepositoryBlox().getDataSourceNames();
        } catch (ServerBloxException e) {
            return false;
        }

        for (int i = 0; i < dataSources.length; i++) {
            if (dataSources[i].equalsIgnoreCase(AppContextImpl.ESSBASE_DATASOURCE_NAME) ||
                    dataSources[i].equalsIgnoreCase(AppContextImpl.MSAS_DATASOURCE_NAME))
                return true;
        }
        return false;
    }

    protected Application createApplication(String contextName, String displayName, String description, String adminRole) throws Exception {
        IWASAdapter wasAdapter = Server.getServer().getWASAdapter();

        if (wasAdapter.exists(contextName, contextName))
            throw new IllegalStateException("Application context " + contextName + " already exists.");

        ApplicationManager appManager = (ApplicationManager)ServiceManager.getService(ServiceManager.SVC_APPMGR, null);

        File docBase = new File(wasAdapter.getWASDefaultAppsHome(), contextName);

        // if WebSphere we need to delete the empty .ear file created by the ApplicationManager so when
        // calling  addWebDescriptor() below it re-creates the .ear with all the contents
        if (isIBMWebSphere()) {
            File earFile = new File(docBase.getParentFile(), contextName + ".ear");

            earFile.delete();
        }

        FileUtils.copyDirectory(appDirectory, docBase);

        // Update the app config file
        File configFile = new File(docBase, CONFIG_FILE);
        AppConfiguration config = new AppConfiguration();
        config.init(this);
        config.setConfigFile(configFile);
        config.readConfig();
        config.setParameter(TITLE, displayName);
        config.setParameter(ADMIN_ROLE, adminRole);
        config.saveConfig();

        // Update web.xml file
        WebDescriptor webDescriptor = wasAdapter.createWebDescriptor(contextName, contextName, "/" + contextName, docBase);
        webDescriptor.setDisplayName(displayName);
        if ((description != null) && (description.length() > 0))
            webDescriptor.setDescription(description);
        webDescriptor.save();

        // Erase log file
        File file = new File(docBase, LOGFILE);
        if (file.exists())
            file.delete();
        LogFile log = new LogFile(displayName, getLocale());
        log.setFile(file);
        log.writeln(LogFile.INFO, getString("APPLICATION_CREATED"));

        wasAdapter.addWebDescriptor(webDescriptor);
        com.alphablox.server.application.Application aasApp = appManager.importAlphabloxApplication(contextName, contextName);

        // This load/save keeps the aas application in sync with the WAS context
        webDescriptor = wasAdapter.getWebDescriptor(contextName, contextName);

        if ((isIBMWebSphere()) && (webDescriptor == null)) {
            WebDescriptor desc = wasAdapter.createWebDescriptor(contextName, contextName, "/" + contextName, docBase);

            wasAdapter.addWebDescriptor(desc);

            aasApp.setDisplayName(desc.getDisplayName());
            aasApp.setDescription(desc.getDescription());

            return null;
        }

        wasAdapter.refreshWebDescriptor(webDescriptor);
        aasApp.load();
        aasApp.save(false, false);

        return new com.alphablox.internal.admin.ApplicationAdminImpl(aasApp.getPrimaryName(), appManager);
    }

    public boolean isIBMWebSphere() {
        return Server.getServer().isIBMWebSphere();
    }

    public void clearLogFile() throws IOException {
        logFile.clear();
        logFile.writeln(LogFile.INFO, getString("LOGFILE_CLEARED"));
    }

    public void validateReportParams(HttpServletRequest request) {
        String reportId = request.getParameter("report");
        if (reportId == null)
            throw new IllegalStateException(getString("ERROR_MISSING_REPORT_PARAMETER"));

        String reportType = request.getParameter("type");

        if (reportType == null || reportType.trim().equals("")) {
            reportType = "public";
        }

        if (!(reportType.equals(Report.PUBLIC) || reportType.equals(Report.PRIVATE) || reportType.equals(Report.GROUP)))
            throw new IllegalArgumentException(getString("ERROR_BAD_REPORT_TYPE"));

        if (reportType.equals("group")) {
            String groupName = request.getParameter("group");
            if (groupName == null || groupName.trim().equals("")) {
                throw new IllegalArgumentException(getString("ERROR_GROUP_REPORT_WITHOUT_GROUP"));
            }
        }
    }

    public ContainerBlox getReportHelpLinkBlox() {
        return reportHelpLink;
    }

    public FastForwardResourceBundle getResourceBundle() {
        return resource;
    }

    class MissingTemplate extends TemplateImpl {
        String name;

        public MissingTemplate(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public String getDisplayName() {
            return "Missing Template: " + name;
        }

        public String getDescription() {
            return "Missing template " + name;
        }

        public Parameter getParameter(String name) {
            return null;
        }

        public Parameter[] getParameters() {
            return new Parameter[0];
        }

        public String getReportURL() {
            return "missingtemplate.jsp";
        }

        public String getPrintPageURL() {
            return null;
        }

        public String getExcelPageURL() {
            return null;
        }

        public void setValidator(ReportValidator validator) {
        }

        public String getEditPageURL() {
            return "../missingtemplate.jsp";
        }


    }
}
