/*
 * Created by IntelliJ IDEA.
 * User: DMESSINK
 * Date: Jun 15, 2004
 * Time: 12:04:39 PM
 */
package fastforward;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.net.URLDecoder;

public class MultipartRequestParser {
    private final String CONTENT_TYPE_PATTERN = "Content-Disposition: form-data; name=\"([^\"]*)\"(?:; filename=\"([^\"]*)\")?";
    private final String CONTENT_TYPE = "Content-Type: ";
    private final byte[] NEWLINE = "\r\n".getBytes("UTF-8");

    private HashMap parameters = new HashMap();
    private HashMap files = new HashMap();

    public MultipartRequestParser(HttpServletRequest request) throws IOException {
        if (request.getContentType().indexOf("multipart/form-data") < 0)
            throw new RuntimeException("MultipartRequestParser can only be used on Content Type multipart/form-data");

        readRequestData(request.getInputStream());
    }

    public String getParameter(String parameterName) {
        return (String) parameters.get(parameterName);
    }

    public File getFile(String parameterName) {
        return (File) files.get(parameterName);
    }

    private void readRequestData(ServletInputStream inputStream) throws IOException {
        String boundary = readLine(inputStream);

        Pattern pattern = Pattern.compile(CONTENT_TYPE_PATTERN);

        while (boundary != null && !boundary.endsWith("--")) {
            String contentHeader = readLine(inputStream);
            Matcher matcher = pattern.matcher(contentHeader);

            if (!matcher.find())
                throw new RuntimeException("Failed to find Content-Disposition");

            String name = matcher.group(1);
            String filename = matcher.group(2);

            if (filename != null) {
                boundary = readFileContents(inputStream, boundary, name, filename);
            } else {
                readToBlankLine(inputStream);

                String value = readLine(inputStream);
                if (value != null)
                    parameters.put(name, URLDecoder.decode(value, "UTF-8"));

                boundary = readLine(inputStream);
            }
        }

        inputStream.close();
    }

    private String readFileContents(ServletInputStream inputStream, String boundary, String name, String filename) throws IOException {
        String contentType = readContentType(inputStream);

        if (contentType == null)
            contentType = "application/octet-stream";
        else
            readToBlankLine(inputStream);

        ByteArrayOutputStream fileContents = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int length = inputStream.readLine(buffer, 0, buffer.length);
        boolean addNewline = false;

        while (length > 0 && !isBoundry(boundary, buffer, length - 2)) {
            // The ServletInputStream will add a newline to the last buffer read - so we need to not write that one
            if (addNewline) {
                fileContents.write(NEWLINE);
                addNewline = false;
            }

            if (endsWithNewline(buffer, length)) {
                addNewline = true;
                length -= 2;
            }

            fileContents.write(buffer, 0, length);
            length = inputStream.readLine(buffer, 0, buffer.length);
        }

        if (fileContents.size() > 0)
            files.put(name, new File(filename, contentType, fileContents.toByteArray()));

        boundary = new String(buffer, 0, length - 2, "UTF-8");
        return boundary;
    }

    private String readContentType(ServletInputStream inputStream) throws IOException {
        String header = readLine(inputStream);

        while (header != null && !header.startsWith(CONTENT_TYPE))
            header = readLine(inputStream);

        return (header == null) ? null : header.substring(CONTENT_TYPE.length());
    }

    private boolean endsWithNewline(byte[] buffer, int length) {
        return (length >= 2) && (buffer[length - 2] == NEWLINE[0] && buffer[length - 1] == NEWLINE[1]);
    }

    private void readToBlankLine(ServletInputStream inputStream) throws IOException {
        String blankLine = readLine(inputStream);

        while (blankLine != null)
            blankLine = readLine(inputStream);
    }

    private boolean isBoundry(String boundary, byte[] buffer, int length) {
        for (int x = 0; x < boundary.length(); x++) {
            if (x >= length)
                return false;

            if (boundary.charAt(x) != buffer[x])
                return false;
        }

        return true;
    }

    private String readLine(ServletInputStream inputStream) throws IOException {
        byte[] buffer = new byte[1024];
        int length = inputStream.readLine(buffer, 0, buffer.length);

        return (length <= 2) ? null : new String(buffer, 0, length - 2, "UTF-8");
    }

    public static class File {
        private String filename;
        private String contentType;
        private byte[] contents;

        public File(String filename, String contentType, byte[] contents) {
            this.filename = filename;
            this.contentType = contentType;
            this.contents = contents;
        }

        public String getFilename() {
            return filename;
        }

        public String getContentType() {
            return contentType;
        }

        public byte[] getContents() {
            return contents;
        }
    }

}