/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: rraghavan
 * Date: Jan 24, 2003 1:03:04 PM
 */
package fastforward;

import com.alphablox.blox.*;
import com.alphablox.blox.filter.BookmarkSaveEvent;
import com.alphablox.blox.filter.BookmarkSaveFilter;
import com.alphablox.blox.form.FormBlox;
import com.alphablox.blox.form.IFormBlox;
import com.alphablox.blox.repository.Bookmark;
import com.alphablox.blox.uimodel.BloxModel;
import com.alphablox.blox.uimodel.core.ModelException;
import com.alphablox.blox.uimodel.core.event.IEventHandler;
import com.alphablox.blox.uimodel.core.event.InitialRenderEvent;

import java.util.Hashtable;
import java.util.Vector;

public class SavedStateImpl implements SavedState, BloxContextEventListener {
    private boolean autoRestore = true;
    private ReportImpl report;  // current report
    private AppContextImpl appContext;
    private Vector bloxList = new Vector();    // list of Blox on the current report
    private String visibility; // visibility of the currently viewed report
    private Hashtable bloxBookmarks = new Hashtable();

    public SavedStateImpl() {
    }

    public SavedStateImpl(AppContextImpl appContext) throws ServerBloxException {
        this.appContext = appContext;
    }

    public void init(ReportImpl report, String visibility) throws ServerBloxException {
        this.report = report;
        this.visibility = visibility;
        bloxList.clear();
        bloxBookmarks.clear();
        appContext.getBloxContext().addBloxContextEventListener(this);
        restoreReportBlox();
    }

    public Blox getBlox(String bloxName) throws ServerBloxException {
        for (int i = 0; i < bloxList.size(); i++) {
            Blox blox = (Blox) bloxList.get(i);
            if (blox.getBloxName().equals(bloxName))
                return blox;
        }
        return null;
    }

    public Blox[] getBlox() {
        return (Blox[]) bloxList.toArray(new Blox[bloxList.size()]);
    }

    public Bookmark getBloxBookmark(String bloxName) throws ServerBloxException {
        if ((report.getSavedStateData() == null) || (visibility == null) || (visibility.equals(Report.PUBLIC)))
            return null;

        Blox blox = getBlox(bloxName);
        if (blox == null || (!(blox instanceof BookmarkableBlox)))
            return null;

        Bookmark bookmark;

        bookmark = (Bookmark) bloxBookmarks.get(bloxName);
        if (bookmark == null) {
            String bookmarkVisibility = visibility.equals(Report.PRIVATE) ? BookmarksBlox.PRIVATE_VISIBILITY : BookmarksBlox.PUBLIC_VISIBILITY;
            String bookmarkName = report.getSavedStateData().getBookmarkName(bloxName);
            bookmark = appContext.getBookmarksBlox().getBookmark(blox, bookmarkName, bookmarkVisibility);
            bloxBookmarks.put(bloxName, bookmark);
        }
        return bookmark;
    }

    public void restore() throws ServerBloxException {
        for (int i = 0; i < bloxList.size(); i++) {
            Blox blox = (Blox) bloxList.get(i);
            restoreBlox(blox);
        }
    }

    public void restoreBlox(Blox blox) throws ServerBloxException {
        if (blox instanceof IFormBlox)
            restoreFormBlox((FormBlox) blox);
        else if (blox instanceof BookmarkableBlox)
            restoreBookmarkableBlox((BookmarkableBlox) blox);
        else if (blox instanceof ReportBlox)
            restoreReportBlox((ReportBlox) blox);
        else
            throw new IllegalArgumentException(appContext.getString("ERROR_WRONG_BLOX_TYPE"));
    }


    private void restoreBookmarkableBlox(BookmarkableBlox blox) throws ServerBloxException {
        if (report.getSavedStateData() == null) return; // nothing saved for this report
        String bmName = report.getSavedStateData().getBookmarkName(((Blox) blox).getBloxName());
        if (bmName != null) {
            Bookmark modifiedBookmark = (Bookmark) bloxBookmarks.get(((Blox) blox).getBloxName());
            if (modifiedBookmark != null) {
                blox.loadBookmark(modifiedBookmark);
            } else {
                blox.loadBookmark(getVisibility(visibility), getOwner(visibility), bmName);
            }
        }
    }

    private void restoreFormBlox(FormBlox blox) throws ServerBloxException {
        if (report.getSavedStateData() == null) return; // nothing saved for this report
        String[] values = report.getSavedStateData().getControlValues(blox.getBloxName());
        blox.setFormValues(values);
        try {
            blox.changed();
        } catch (ModelException e) {
            throw new ServerBloxException(e);
        }
    }

    private void restoreReportBlox() throws ServerBloxException {
        if (report == null || report.getSavedStateData() == null)
            return;

        SavedStateData savedStateData = report.getSavedStateData();

        SavedStateData.Bookmark[] bookmarks = savedStateData.getReportBloxBookmarks();
        for (int x = 0; x < bookmarks.length; x++) {
            SavedStateData.Bookmark bookmark = bookmarks[x];

            ReportBlox reportBlox = new ReportBlox();
            reportBlox.init(appContext.getBloxContext(), bookmark.getBloxName());
            restoreReportBlox(reportBlox);
        }
    }

    private void restoreReportBlox(ReportBlox reportBlox) throws ServerBloxException {
        if (report.getSavedStateData() == null)
            return;

        String bookmarkName = report.getSavedStateData().getBookmarkName(reportBlox.getBloxName());

        reportBlox.loadBookmark(getVisibility(visibility), getOwner(visibility), bookmarkName);
    }

    public void setAutoRestore(boolean autoRestore) {
        this.autoRestore = autoRestore;
    }

    public boolean isAutoRestore() {
        return autoRestore;
    }

    // non-interface methods
    public ReportImpl getReport() {
        return report;
    }

    public void saveReportAs(String reportLabel, String reportDesc, String visibility, String groupName) throws Exception {
        IConfiguration config;
        if (visibility.equals(Report.PRIVATE))
            config = appContext.getUserConfiguration();
        else
            config = appContext.getGroupConfiguration(groupName);

        appContext.getUIControl().createReport(this.report, reportLabel, reportDesc, null, this, config);
    }

    public void save(String reportID, SavedStateData savedStateData, String visibility) throws ServerBloxException {
        for (int i = 0; i < bloxList.size(); i++) {
            Blox blox = (Blox) bloxList.elementAt(i);
            if (blox instanceof IFormBlox)
                saveFormBlox((FormBlox) blox, savedStateData);
            else if (blox instanceof BookmarkableBlox)
                saveBookmarkableBlox(reportID, (BookmarkableBlox) blox, savedStateData, visibility);
            else if (blox instanceof ReportBlox)
                saveReportBlox(reportID, (ReportBlox) blox, savedStateData, visibility);
        }
    }

    protected void saveBookmarkableBlox(String reportID, BookmarkableBlox blox, SavedStateData savedStateData, String visibility) throws ServerBloxException {
        String bmName = reportID + "_" + ((Blox) blox).getBloxName();

        blox.addBookmarkSaveFilter(new BookmarkSaveFilter() {
            public void bookmarkSave(BookmarkSaveEvent event) throws Exception {
                Bookmark bookmark = event.getBookmark();
                bookmark.setHidden(true);
            }
        });

        blox.saveBookmark(getVisibility(visibility), getOwner(visibility), bmName, "");

        savedStateData.setBookmarkName(((Blox) blox).getBloxName(), bmName);
    }

    protected void saveFormBlox(FormBlox blox, SavedStateData savedState) throws ServerBloxException {
        savedState.setControlValue(blox.getBloxName(), blox.getFormValues());
    }

    private void saveReportBlox(String reportID, ReportBlox reportBlox, SavedStateData savedStateData, String visibility) throws ServerBloxException {
        String bookmarkName = reportID + '_' + reportBlox.getBloxName()+".xml";

        reportBlox.saveBookmark(getVisibility(visibility), getOwner(visibility), bookmarkName, "");

        savedStateData.setBookmarkName(reportBlox.getBloxName(), bookmarkName);
        savedStateData.getBookmark(reportBlox.getBloxName()).setReportBlox(true);
    }

    public void delete() {
        if (report == null)
            return; // Report was deleted via the admin pages

        SavedStateData savedStateData = report.getSavedStateData();

        if (savedStateData == null)
            return; // no bookmarks to delete

        String bmName = "";
        for (int i = 0; i < bloxList.size(); i++) {
            Blox blox = (Blox) bloxList.elementAt(i);
            try {
                bmName = savedStateData.getBookmarkName(blox.getBloxName());

                if (blox instanceof BookmarkableBlox)
                    deleteBookmark((BookmarkableBlox)blox, visibility, bmName);
                else if (blox instanceof ReportBlox)
                    deleteBookmark((ReportBlox)blox, visibility, bmName);

            } catch (ServerBloxException e) {
                appContext.log(LogFile.ERROR, "Error in deleting bookmark " + bmName, e);
            }
        }
    }

    private void deleteBookmark(BookmarkableBlox blox, String visibility, String bmName) throws ServerBloxException {
        String bookmarkVisibility = (visibility.equals(Report.PRIVATE)) ? Bookmark.PRIVATE_VISIBILITY : Bookmark.PUBLIC_VISIBILITY;

        Bookmark bookmark = appContext.getBookmarksBlox().getBookmark((Blox)blox, bmName, bookmarkVisibility);
        if (bookmark != null)
            bookmark.delete();
    }

    private void deleteBookmark(ReportBlox reportBlox, String visibility, String bookmarkName) throws ServerBloxException {
        reportBlox.deleteBookmark(getVisibility(visibility), getOwner(visibility), bookmarkName);
    }

    private int getVisibility(String visibility) {
        if (visibility.equals(Report.PRIVATE))
            return RepositoryBlox.VISIBILITY_PRIVATE;
        else
            return RepositoryBlox.VISIBILITY_APPLICATION;
    }

    private String getOwner(String visibility) {
        if (visibility.equals(Report.PRIVATE))
            return appContext.getUser().getName();
        else
            return appContext.getContextPath();
    }

    // implementing the BloxContextEventListener interface
    public void bloxCreated(BloxContextEvent event) {
        Blox blox = event.getBlox();
        String bloxName = blox.getBloxName();

        if (bloxName == null || bloxName.startsWith("ffwd") || (!blox.isTopLevel()))
            return;

        bloxList.add(blox);
        if (blox instanceof ContainerBlox) {
            ContainerBlox containerBlox = (ContainerBlox) blox;
            try {
                BloxModel model = containerBlox.getBloxModel();
                if (getReport() != null) {
                    SavedStateData savedData = getReport().getSavedStateData();
                    if (savedData != null) {
                        if (savedData.getBookmarkName(bloxName) != null ||
                                savedData.getControlValue(bloxName) != null)
                            model.addEventHandler(new RenderEventHandler(this, containerBlox));
                    }
                }
            } catch (ServerBloxException e) {
                e.printStackTrace();  //To change body of catch statement use Options | File Templates.
            }
        }
    }

    public void bloxDestroyed(BloxContextEvent event) {
        bloxList.remove(event.getBlox());
    }
    // end of implementing the BloxContextEventListener interface

    class RenderEventHandler implements IEventHandler {
        SavedStateImpl handler;
        Blox blox;

        public RenderEventHandler(SavedStateImpl handler, Blox blox) {
            this.handler = handler;
            this.blox = blox;
        }

        public boolean handleInitialRenderEvent(InitialRenderEvent event) {
            // restore the bookmark here
            if (!handler.isAutoRestore()) return false;

            String name = "";
            try {
                name = blox.getBloxName();
                restoreBlox(blox);
            } catch (ServerBloxException e) {
                appContext.log(LogFile.ERROR, appContext.getString("ERROR_RESTORE_BOOKMARK", new Object[]{name}), e);
                e.printStackTrace();
            }
            return false;
        }
    }
}

